@@ -1,11 +1,11 @@
 # pylint: disable=bad-option-value,wrong-import-position
 # We need to disable the import position checks because of the windows check that we need to do below
+import copy
 import logging
 import os
 import platform
-import select
+import stat
 import sys
-
 import click
 
 # Error codes
@@ -14,13 +14,6 @@ USAGE_ERROR_CODE = 253  # noqa
 GIT_CONTEXT_ERROR_CODE = 254  # noqa
 CONFIG_ERROR_CODE = 255  # noqa
 
-# We need to make sure we're not on Windows before importing other gitlint modules, as some of these modules use sh
-# which will raise an exception when imported on Windows.
-if "windows" in platform.system().lower():  # noqa
-    click.echo("Gitlint currently does not support Windows. Check out "  # noqa
-               "https://github.com/jorisroovers/gitlint/issues/20 for details.", err=True)  # noqa
-    exit(USAGE_ERROR_CODE)  # noqa
-
 import gitlint
 from gitlint.lint import GitLinter
 from gitlint.config import LintConfigBuilder, LintConfigError, LintConfigGenerator
@@ -52,9 +45,12 @@ def log_system_info():
     LOG.debug("Python version: %s", sys.version)
     LOG.debug("Git version: %s", git_version())
     LOG.debug("Gitlint version: %s", gitlint.__version__)
+    LOG.debug("GITLINT_USE_SH_LIB: %s", os.environ.get("GITLINT_USE_SH_LIB", "[NOT SET]"))
 
 
-def build_config(ctx, target, config_path, c, extra_path, ignore, verbose, silent, debug):
+def build_config(  # pylint: disable=too-many-arguments
+        ctx, target, config_path, c, extra_path, ignore, contrib, ignore_stdin, verbose, silent, debug
+):
     """ Creates a LintConfig object based on a set of commandline parameters. """
     config_builder = LintConfigBuilder()
     try:
@@ -71,6 +67,13 @@ def build_config(ctx, target, config_path, c, extra_path, ignore, verbose, silen
         # Finally, overwrite with any convenience commandline flags
         if ignore:
             config_builder.set_option('general', 'ignore', ignore)
+
+        if contrib:
+            config_builder.set_option('general', 'contrib', contrib)
+
+        if ignore_stdin:
+            config_builder.set_option('general', 'ignore-stdin', ignore_stdin)
+
         if silent:
             config_builder.set_option('general', 'verbosity', 0)
         elif verbose > 0:
@@ -93,20 +96,61 @@ def build_config(ctx, target, config_path, c, extra_path, ignore, verbose, silen
     ctx.exit(CONFIG_ERROR_CODE)  # return CONFIG_ERROR_CODE on config error
 
 
-def stdin_has_data():
-    """ Helper function that indicates whether the stdin has data incoming or not """
-    # This code was taken from:
-    # https://stackoverflow.com/questions/3762881/how-do-i-check-if-stdin-has-some-data
+def get_stdin_data():
+    """ Helper function that returns data send to stdin or False if nothing is send """
+    # STDIN can only be 3 different types of things ("modes")
+    #  1. An interactive terminal device (i.e. a TTY -> sys.stdin.isatty() or stat.S_ISCHR)
+    #  2. A (named) pipe (stat.S_ISFIFO)
+    #  3. A regular file (stat.S_ISREG)
+    # Technically, STDIN can also be other device type like a named unix socket (stat.S_ISSOCK), but we don't
+    # support that in gitlint (at least not today).
+    #
+    # Now, the behavior that we want is the following:
+    # If someone sends something directly to gitlint via a pipe or a regular file, read it. If not, read from the
+    # local repository.
+    # Note that we don't care about whether STDIN is a TTY or not, we only care whether data is via a pipe or regular
+    # file.
+    # However, in case STDIN is not a TTY, it HAS to be one of the 2 other things (pipe or regular file), even if
+    # no-one is actually sending anything to gitlint over them. In this case, we still want to read from the local
+    # repository.
+    # To support this use-case (which is common in CI runners such as Jenkins and Gitlab), we need to actually attempt
+    # to read from STDIN in case it's a pipe or regular file. In case that fails, then we'll fall back to reading
+    # from the local repo.
 
-    # Caveat, this probably doesn't work on Windows because the code is dependent on the unix SELECT syscall.
-    # Details: https://docs.python.org/2/library/select.html#select.select
-    # This isn't a real problem now, because gitlint as a whole doesn't support Windows (see #20).
-    # If we ever do, we probably want to fall back to the old detection mechanism of reading from the local git repo
-    # in case there's no TTY connected to STDIN.
-    return select.select([sys.stdin, ], [], [], 0.0)[0]
+    mode = os.fstat(sys.stdin.fileno()).st_mode
+    stdin_is_pipe_or_file = stat.S_ISFIFO(mode) or stat.S_ISREG(mode)
+    if stdin_is_pipe_or_file:
+        input_data = sys.stdin.read()
+        # Only return the input data if there's actually something passed
+        # i.e. don't consider empty piped data
+        if input_data:
+            return ustr(input_data)
+    return False
 
 
-@click.group(invoke_without_command=True, epilog="When no COMMAND is specified, gitlint defaults to 'gitlint lint'.")
+def build_git_context(lint_config, msg_filename, refspec):
+    """ Builds a git context based on passed parameters and order of precedence """
+    # Order of precedence:
+    # 1. Any data specified via --msg-filename
+    if msg_filename:
+        LOG.debug("Attempting to read from --msg-filename.")
+        return GitContext.from_commit_msg(ustr(msg_filename.read()))
+
+    # 2. Any data sent to stdin (unless stdin is being ignored)
+    if not lint_config.ignore_stdin:
+        stdin_input = get_stdin_data()
+        if stdin_input:
+            LOG.debug("Stdin data: %r", stdin_input)
+            LOG.debug("Stdin detected and not ignored. Will be used as input.")
+            return GitContext.from_commit_msg(stdin_input)
+
+    # 3. Fallback to reading from local repository
+    LOG.debug("No --msg-filename flag, no or empty data passed to stdin. Attempting to read from the local repo.")
+    return GitContext.from_local_repository(lint_config.target, refspec)
+
+
+@click.group(invoke_without_command=True, context_settings={'max_content_width': 120},
+             epilog="When no COMMAND is specified, gitlint defaults to 'gitlint lint'.")
 @click.option('--target', type=click.Path(exists=True, resolve_path=True, file_okay=False, readable=True),
               help="Path of the target git repository. [default: current working directory]")
 @click.option('-C', '--config', type=click.Path(exists=True, dir_okay=False, readable=True, resolve_path=True),
@@ -118,28 +162,38 @@ def stdin_has_data():
 @click.option('-e', '--extra-path', help="Path to a directory or python module with extra user-defined rules",
               type=click.Path(exists=True, resolve_path=True, readable=True))
 @click.option('--ignore', default="", help="Ignore rules (comma-separated by id or name).")
+@click.option('--contrib', default="", help="Contrib rules to enable (comma-separated by id or name).")
+@click.option('--msg-filename', type=click.File(), help="Path to a file containing a commit-msg.")
+@click.option('--ignore-stdin', is_flag=True, help="Ignore any stdin data. Useful for running in CI server.")
 @click.option('-v', '--verbose', count=True, default=0,
               help="Verbosity, more v's for more verbose output (e.g.: -v, -vv, -vvv). [default: -vvv]", )
 @click.option('-s', '--silent', help="Silent mode (no output). Takes precedence over -v, -vv, -vvv.", is_flag=True)
 @click.option('-d', '--debug', help="Enable debugging output.", is_flag=True)
 @click.version_option(version=gitlint.__version__)
 @click.pass_context
-def cli(ctx, target, config, c, commits, extra_path, ignore, verbose, silent, debug):
-    """ Git lint tool, checks your git commit messages for styling issues """
+def cli(  # pylint: disable=too-many-arguments
+        ctx, target, config, c, commits, extra_path, ignore, contrib,
+        msg_filename, ignore_stdin, verbose, silent, debug,
+):
+    """ Git lint tool, checks your git commit messages for styling issues
+
+        Documentation: http://jorisroovers.github.io/gitlint
+    """
 
     try:
         if debug:
             logging.getLogger("gitlint").setLevel(logging.DEBUG)
+        LOG.debug("To report issues, please visit https://github.com/jorisroovers/gitlint/issues")
 
         log_system_info()
 
         # Get the lint config from the commandline parameters and
         # store it in the context (click allows storing an arbitrary object in ctx.obj).
-        config, config_builder = build_config(ctx, target, config, c, extra_path, ignore, verbose, silent, debug)
-
+        config, config_builder = build_config(ctx, target, config, c, extra_path,
+                                              ignore, contrib, ignore_stdin, verbose, silent, debug)
         LOG.debug(u"Configuration\n%s", ustr(config))
 
-        ctx.obj = (config, config_builder, commits)
+        ctx.obj = (config, config_builder, commits, msg_filename)
 
         # If no subcommand is specified, then just lint
         if ctx.invoked_subcommand is None:
@@ -155,20 +209,17 @@ def cli(ctx, target, config, c, commits, extra_path, ignore, verbose, silent, de
 def lint(ctx):
     """ Lints a git repository [default command] """
     lint_config = ctx.obj[0]
+    refspec = ctx.obj[2]
+    msg_filename = ctx.obj[3]
 
-    # If we get data via stdin, then let's consider that our commit message, otherwise parse it from the local git repo.
-    if stdin_has_data():
-        stdin_str = ustr(sys.stdin.read())
-        gitcontext = GitContext.from_commit_msg(stdin_str)
-    else:
-        gitcontext = GitContext.from_local_repository(lint_config.target, ctx.obj[2])
+    gitcontext = build_git_context(lint_config, msg_filename, refspec)
 
     number_of_commits = len(gitcontext.commits)
     # Exit if we don't have commits in the specified range. Use a 0 exit code, since a popular use-case is one
     # where users are using --commits in a check job to check the commit messages inside a CI job. By returning 0, we
     # ensure that these jobs don't fail if for whatever reason the specified commit range is empty.
     if number_of_commits == 0:
-        click.echo(u'No commits in range "{0}".'.format(ctx.obj[2]))
+        LOG.debug(u'No commits in range "%s"', ctx.obj[2])
         ctx.exit(0)
 
     general_config_builder = ctx.obj[1]
@@ -178,13 +229,16 @@ def lint(ctx):
     first_violation = True
     exit_code = 0
     for commit in gitcontext.commits:
-        # Build a config_builder and linter taking into account the commit specific config (if any)
+        # Build a config_builder taking into account the commit specific config (if any)
         config_builder = general_config_builder.clone()
         config_builder.set_config_from_commit(commit)
-        lint_config = config_builder.build(lint_config)
-        linter = GitLinter(lint_config)
+
+        # Create a deepcopy from the original config, so we have a unique config object per commit
+        # This is important for configuration rules to be able to modifying the config on a per commit basis
+        commit_config = config_builder.build(copy.deepcopy(lint_config))
 
         # Actually do the linting
+        linter = GitLinter(commit_config)
         violations = linter.lint(commit)
         # exit code equals the total number of violations in all commits
         exit_code += len(violations)
@@ -242,7 +296,7 @@ def uninstall_hook(ctx):
 def generate_config(ctx):
     """ Generates a sample gitlint config file. """
     path = click.prompt('Please specify a location for the sample gitlint config file', default=DEFAULT_CONFIG_FILE)
-    path = os.path.abspath(path)
+    path = os.path.realpath(path)
     dir_name = os.path.dirname(path)
     if not os.path.exists(dir_name):
         click.echo(u"Error: Directory '{0}' does not exist.".format(dir_name), err=True)
@@ -259,4 +313,5 @@ def generate_config(ctx):
 # Let's Party!
 setup_logging()
 if __name__ == "__main__":
-    cli()  # pragma: no cover, # pylint: disable=no-value-for-parameter
+    # pylint: disable=no-value-for-parameter
+    cli()  # pragma: no cover
