@@ -6,21 +6,17 @@ except ImportError:  # pragma: no cover
     from configparser import ConfigParser, Error as ConfigParserError  # pragma: no cover, pylint: disable=import-error
 
 import copy
+import io
 import re
 import os
 import shutil
 
-try:
-    # python >= 2.7
-    from collections import OrderedDict  # pylint: disable=no-name-in-module
-except ImportError:  # pragma: no cover
-    # python 2.4-2.6
-    from ordereddict import OrderedDict  # pragma: no cover
-
-from gitlint.utils import ustr
+from collections import OrderedDict
+from gitlint.utils import ustr, DEFAULT_ENCODING
 from gitlint import rules  # For some weird reason pylint complains about this, pylint: disable=unused-import
 from gitlint import options
-from gitlint import user_rules
+from gitlint import rule_finder
+from gitlint.contrib import rules as contrib_rules
 
 
 def handle_option_error(func):
@@ -46,7 +42,9 @@ class LintConfig(object):
     """
 
     # Default tuple of rule classes (tuple because immutable).
-    default_rule_classes = (rules.TitleMaxLength,
+    default_rule_classes = (rules.IgnoreByTitle,
+                            rules.IgnoreByBody,
+                            rules.TitleMaxLength,
                             rules.TitleTrailingWhitespace,
                             rules.TitleLeadingWhitespace,
                             rules.TitleTrailingPunctuation,
@@ -72,9 +70,12 @@ class LintConfig(object):
         self._debug = options.BoolOption('debug', False, "Enable debug mode")
         self._extra_path = None
         target_description = "Path of the target git repository (default=current working directory)"
-        self._target = options.PathOption('target', os.path.abspath(os.getcwd()), target_description)
+        self._target = options.PathOption('target', os.path.realpath(os.getcwd()), target_description)
         self._ignore = options.ListOption('ignore', [], 'List of rule-ids to ignore')
+        self._contrib = options.ListOption('contrib', [], 'List of contrib-rules to enable')
         self._config_path = None
+        ignore_stdin_description = "Ignore any stdin data. Useful for running in CI server."
+        self._ignore_stdin = options.BoolOption('ignore-stdin', False, ignore_stdin_description)
 
     @property
     def target(self):
@@ -150,19 +151,19 @@ class LintConfig(object):
 
             # Make sure we unload any previously loaded extra-path rules
             for rule in self.rules:
-                if hasattr(rule, 'user_defined') and rule.user_defined:
+                if hasattr(rule, 'is_user_defined') and rule.is_user_defined:
                     del self._rules[rule.id]
 
             # Find rules in the new extra-path
-            rule_classes = user_rules.find_rule_classes(self.extra_path)
+            rule_classes = rule_finder.find_rule_classes(self.extra_path)
 
             # Add the newly found rules to the existing rules
             for rule_class in rule_classes:
                 rule_obj = rule_class()
-                rule_obj.user_defined = True
+                rule_obj.is_user_defined = True
                 self._rules[rule_class.id] = rule_obj
 
-        except (options.RuleOptionError, user_rules.UserRuleError) as e:
+        except (options.RuleOptionError, rules.UserRuleError) as e:
             raise LintConfigError(ustr(e))
 
     @property
@@ -175,6 +176,49 @@ class LintConfig(object):
             value = [rule.id for rule in self.rules]
         return self._ignore.set(value)
 
+    @property
+    def contrib(self):
+        return self._contrib.value
+
+    @contrib.setter
+    def contrib(self, value):
+        try:
+            self._contrib.set(value)
+
+            # Make sure we unload any previously loaded contrib rules when re-setting the value
+            for rule in self.rules:
+                if hasattr(rule, 'is_contrib') and rule.is_contrib:
+                    del self._rules[rule.id]
+
+            # Load all classes from the contrib directory
+            contrib_dir_path = os.path.dirname(os.path.realpath(contrib_rules.__file__))
+            rule_classes = rule_finder.find_rule_classes(contrib_dir_path)
+
+            # For each specified contrib rule, check whether it exists among the contrib classes
+            for rule_id_or_name in self.contrib:
+                rule_class = next((rc for rc in rule_classes if
+                                   rc.id == ustr(rule_id_or_name) or rc.name == ustr(rule_id_or_name)), False)
+
+                # If contrib rule exists, instantiate it and add it to the rules list
+                if rule_class:
+                    rule_obj = rule_class()
+                    rule_obj.is_contrib = True
+                    self._rules[rule_class.id] = rule_obj
+                else:
+                    raise LintConfigError(u"No contrib rule with id or name '{0}' found.".format(ustr(rule_id_or_name)))
+
+        except (options.RuleOptionError, rules.UserRuleError) as e:
+            raise LintConfigError(ustr(e))
+
+    @property
+    def ignore_stdin(self):
+        return self._ignore_stdin.value
+
+    @ignore_stdin.setter
+    @handle_option_error
+    def ignore_stdin(self, value):
+        return self._ignore_stdin.set(value)
+
     @property
     def rules(self):
         # Create a new list based on _rules.values() because in python 3, values() is a ValuesView as opposed to a list
@@ -223,8 +267,9 @@ class LintConfig(object):
         # only allow setting general options that exist and don't start with an underscore
         if not hasattr(self, attr_name) or attr_name[0] == "_":
             raise LintConfigError(u"'{0}' is not a valid gitlint option".format(option_name))
-        else:
-            setattr(self, attr_name, option_value)
+
+        # else:
+        setattr(self, attr_name, option_value)
 
     def __eq__(self, other):
         return isinstance(other, LintConfig) and \
@@ -232,9 +277,11 @@ class LintConfig(object):
                self.verbosity == other.verbosity and \
                self.target == other.target and \
                self.extra_path == other.extra_path and \
+               self.contrib == other.contrib and \
                self.ignore_merge_commits == other.ignore_merge_commits and \
                self.ignore_fixup_commits == other.ignore_fixup_commits and \
                self.ignore_squash_commits == other.ignore_squash_commits and \
+               self.ignore_stdin == other.ignore_stdin and \
                self.debug == other.debug and \
                self.ignore == other.ignore and \
                self._config_path == other._config_path  # noqa
@@ -244,17 +291,19 @@ class LintConfig(object):
         return_str = u"config-path: {0}\n".format(self._config_path)
         return_str += u"[GENERAL]\n"
         return_str += u"extra-path: {0}\n".format(self.extra_path)
+        return_str += u"contrib: {0}\n".format(self.contrib)
         return_str += u"ignore: {0}\n".format(",".join(self.ignore))
         return_str += u"ignore-merge-commits: {0}\n".format(self.ignore_merge_commits)
         return_str += u"ignore-fixup-commits: {0}\n".format(self.ignore_fixup_commits)
         return_str += u"ignore-squash-commits: {0}\n".format(self.ignore_squash_commits)
+        return_str += u"ignore-stdin: {0}\n".format(self.ignore_stdin)
         return_str += u"verbosity: {0}\n".format(self.verbosity)
         return_str += u"debug: {0}\n".format(self.debug)
         return_str += u"target: {0}\n".format(self.target)
         return_str += u"[RULES]\n"
         for rule in self.rules:
             return_str += u"  {0}: {1}\n".format(rule.id, rule.name)
-            for option_name, option_value in rule.options.items():
+            for option_name, option_value in sorted(rule.options.items()):
                 if isinstance(option_value.value, list):
                     option_val_repr = ",".join(option_value.value)
                 else:
@@ -308,10 +357,13 @@ class LintConfigBuilder(object):
         """ Loads lint config from a ini-style config file """
         if not os.path.exists(filename):
             raise LintConfigError(u"Invalid file path: {0}".format(filename))
-        self._config_path = os.path.abspath(filename)
+        self._config_path = os.path.realpath(filename)
         try:
             parser = ConfigParser()
-            parser.read(filename)
+
+            with io.open(filename, encoding=DEFAULT_ENCODING) as config_file:
+                # readfp() is deprecated in python 3.2+, but compatible with 2.7
+                parser.readfp(config_file, filename)  # pylint: disable=deprecated-method
 
             for section_name in parser.sections():
                 for option_name, option_value in parser.items(section_name):
@@ -353,7 +405,7 @@ class LintConfigBuilder(object):
         return builder
 
 
-GITLINT_CONFIG_TEMPLATE_SRC_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "files/gitlint")
+GITLINT_CONFIG_TEMPLATE_SRC_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "files/gitlint")
 
 
 class LintConfigGenerator(object):
