@@ -1,7 +1,9 @@
+import os
 import arrow
-import sh
+
+from gitlint import shell as sh
 # import exceptions separately, this makes it a little easier to mock them out in the unit tests
-from sh import CommandNotFound, ErrorReturnCode
+from gitlint.shell import CommandNotFound, ErrorReturnCode
 
 from gitlint.utils import ustr, sstr
 
@@ -20,11 +22,10 @@ class GitNotInstalledError(GitContextError):
 
 def _git(*command_parts, **kwargs):
     """ Convenience function for running git commands. Automatically deals with exceptions and unicode. """
-    # Special arguments passed to sh: http://amoffat.github.io/sh/special_arguments.html
     git_kwargs = {'_tty_out': False}
     git_kwargs.update(kwargs)
     try:
-        result = sh.git(*command_parts, **git_kwargs)
+        result = sh.git(*command_parts, **git_kwargs)  # pylint: disable=unexpected-keyword-arg
         # If we reach this point and the result has an exit_code that is larger than 0, this means that we didn't
         # get an exception (which is the default sh behavior for non-zero exit codes) and so the user is expecting
         # a non-zero exit code -> just return the entire result
@@ -35,7 +36,7 @@ def _git(*command_parts, **kwargs):
         raise GitNotInstalledError()
     except ErrorReturnCode as e:  # Something went wrong while executing the git command
         error_msg = e.stderr.strip()
-        if '_cwd' in git_kwargs and b"Not a git repository" in error_msg:
+        if '_cwd' in git_kwargs and b"not a git repository" in error_msg.lower():
             error_msg = u"{0} is not a git repository.".format(git_kwargs['_cwd'])
         else:
             error_msg = u"An error occurred while executing '{0}': {1}".format(e.full_cmd, error_msg)
@@ -47,44 +48,51 @@ def git_version():
     return _git("--version").replace(u"\n", u"")
 
 
-def git_commentchar():
+def git_commentchar(repository_path=None):
     """ Shortcut for retrieving comment char from git config """
-    commentchar = _git("config", "--get", "core.commentchar", _ok_code=[1])
+    commentchar = _git("config", "--get", "core.commentchar", _cwd=repository_path, _ok_code=[0, 1])
     # git will return an exit code of 1 if it can't find a config value, in this case we fall-back to # as commentchar
-    if commentchar.exit_code == 1:  # pylint: disable=no-member
+    if hasattr(commentchar, 'exit_code') and commentchar.exit_code == 1:  # pylint: disable=no-member
         commentchar = "#"
     return ustr(commentchar).replace(u"\n", u"")
 
 
+def git_hooks_dir(repository_path):
+    """ Determine hooks directory for a given target dir """
+    hooks_dir = _git("rev-parse", "--git-path", "hooks", _cwd=repository_path)
+    hooks_dir = ustr(hooks_dir).replace(u"\n", u"")
+    return os.path.realpath(os.path.join(repository_path, hooks_dir))
+
+
 class GitCommitMessage(object):
     """ Class representing a git commit message. A commit message consists of the following:
+      - context: The `GitContext` this commit message is part of
       - original: The actual commit message as returned by `git log`
       - full: original, but stripped of any comments
       - title: the first line of full
       - body: all lines following the title
     """
-    COMMENT_CHAR = git_commentchar()
-    CUTLINE = '{0} ------------------------ >8 ------------------------'.format(COMMENT_CHAR)
-
-    def __init__(self, original=None, full=None, title=None, body=None):
+    def __init__(self, context, original=None, full=None, title=None, body=None):
+        self.context = context
         self.original = original
         self.full = full
         self.title = title
         self.body = body
 
     @staticmethod
-    def from_full_message(commit_msg_str):
+    def from_full_message(context, commit_msg_str):
         """  Parses a full git commit message by parsing a given string into the different parts of a commit message """
         all_lines = commit_msg_str.splitlines()
+        cutline = u"{0} ------------------------ >8 ------------------------".format(context.commentchar)
         try:
-            cutline_index = all_lines.index(GitCommitMessage.CUTLINE)
+            cutline_index = all_lines.index(cutline)
         except ValueError:
             cutline_index = None
-        lines = [line for line in all_lines[:cutline_index] if not line.startswith(GitCommitMessage.COMMENT_CHAR)]
+        lines = [ustr(line) for line in all_lines[:cutline_index] if not line.startswith(context.commentchar)]
         full = "\n".join(lines)
-        title = lines[0] if len(lines) > 0 else ""
+        title = lines[0] if lines else ""
         body = lines[1:] if len(lines) > 1 else []
-        return GitCommitMessage(original=commit_msg_str, full=full, title=title, body=body)
+        return GitCommitMessage(context=context, original=commit_msg_str, full=full, title=title, body=body)
 
     def __unicode__(self):
         return self.full  # pragma: no cover
@@ -149,8 +157,10 @@ class GitContext(object):
     the git repository that gitlint is linting.
     """
 
-    def __init__(self):
+    def __init__(self, repository_path=None):
         self.commits = []
+        self.repository_path = repository_path
+        self.commentchar = git_commentchar(repository_path)
 
     @staticmethod
     def from_commit_msg(commit_msg_str):
@@ -158,7 +168,7 @@ class GitContext(object):
         :param commit_msg_str: Full git commit message.
         """
         context = GitContext()
-        commit_msg_obj = GitCommitMessage.from_full_message(commit_msg_str)
+        commit_msg_obj = GitCommitMessage.from_full_message(context, commit_msg_str)
 
         commit = GitCommit(context, commit_msg_obj)
 
@@ -172,7 +182,7 @@ class GitContext(object):
         :param refspec: The commit(s) to retrieve
         """
 
-        context = GitContext()
+        context = GitContext(repository_path=repository_path)
 
         # If no refspec is defined, fallback to the last commit on the current branch
         if refspec is None:
@@ -202,7 +212,7 @@ class GitContext(object):
             commit_date = arrow.get(ustr(date), "YYYY-MM-DD HH:mm:ss Z").datetime
 
             # Create Git commit object with the retrieved info
-            commit_msg_obj = GitCommitMessage.from_full_message(commit_msg)
+            commit_msg_obj = GitCommitMessage.from_full_message(context, commit_msg)
 
             commit = GitCommit(context, commit_msg_obj, sha=sha, author_name=name,
                                author_email=email, date=commit_date, changed_files=changed_files,
